#!/usr/bin/env bash
#
# backup-rotate
# delete the oldest files from a set, only leave a desired amount
# Copyright (c) 2006-2011, Matej Sustr
# 
# $Id: backup-rotate.sh 33 2011-01-12 23:02:05Z matej $
VERSION="1.8"
COPYRIGHT="Copyright (c) 2006-2011, Matej Sustr"
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

usage() { if [ "x${LANG:0:2}" != "xsk" ]; then cat <<. | cut -c 3-
# 
# backup-rotate v$VERSION - delete old files     $COPYRIGHT
# ARGUMENTS:
# [-p] [-v] [-d <days>] [-n <nfiles>] <prefix1> [... <prefixN>]
# 	-p		(testing) pretend only, do not delete; best with -v
# 	-v		(verbose) prints every file name deletion
# 	-d <days>	only delete files older than 'days' days
# 			(default = $defdays)
# 	-n <nfiles>	regardless of their age, only delete files so that
# 			at least 'nfiles' are left
# 			(default = $defnfiles)
# 	<prefixK>	prefix of the group of files to delete from
#
# EXAMPLE:
#   $0 -v /var/recordings/  # directories must have a slash!
#   $0 -d 10 /tmp/phpsess-
#   $0 -d 5 -n 5 /var/log/messages. /var/log/syslog.
# 
.
else cat <<. | cut -c 3-
# 
# backup-rotate v$VERSION - zmaz stare subory    $COPYRIGHT
# PARAMETRE:
# [-p] [-v] [-d <days>] [-n <nfiles>] <prefix1> [... <prefixN>]
# 	-p		(testovanie) len simuluj, nemaz nic; najlepsie s -v
#  	-v		(verbose) vypise kazdy subor, ktory sa bude mazat
# 	-d <days>	bude mazat iba subory starsie ako 'days' dni
# 			(default = $defdays)
# 	-n <nfiles>	bez ohladu na cas, maze subory vzdy tak, aby zostalo
# 			aspon 'nfiles' suborov
# 			(default = $defnfiles)
# 	<prefixK>	prefixy skupiny suborov, ktora sa uvazuje na premazanie
#
# NAPRIKLAD:
#   $0 -v /var/recordings/  # adresar musi mat vzdy lomitko!
#   $0 -d 10 /tmp/phpsess-
#   $0 -d 5 -n 5 /var/log/messages. /var/log/syslog.
# 
.
fi; }
# REQUIRED:
# find, sed, rm, sort
# 
# KNOWN BUGS:
# - undefined behavior if any of the matching files contain \n in their name
# 

MYSELF=backup-rotate
PATH=/usr/local/bin:/usr/bin:/bin	# be safe

##############################################################################

require_number () {
	if ! [ "$OPTARG" -eq "$OPTARG" 2>/dev/null ]; then
		echo >&2 "-$opt needs a number"
		usage
		exit 1
	fi
}

# defaults:
pretend=no
verbose=no
defdays=180;	days=$defdays
defnfiles=6;	nfiles=$defnfiles
while getopts "hpvd:n:" opt; do case $opt in
	p)	pretend=yes;;
	v)	verbose=yes;;
	d)	require_number
		days=$OPTARG;;
	n)	require_number
		nfiles=$OPTARG;;
	h|\?|*)	usage; exit 1;;
esac done
shift $((OPTIND-1))
if [ $# -eq 0 ]; then
	usage; exit 1
fi
if [ "$days" -lt 0 ]; then
	mtime=""		# allow today's files to be deleted
else
	mtime="-mtime +$days"
fi
if [ "$nfiles" -lt 1 ]; then
	nfilesd=""
else
	nfilesd="1,${nfiles}d;"	# keep newest N files
fi



while [ "$1" != "" ]; do
	prefixdir="$(echo "$1" | sed 's/[^\/]*$//')"
	if [ "$prefixdir" = "" ]; then
		prefixdir="."
	fi
	prefixfile="$(echo "$1" | sed 's/.*\///')"
	shift

	find "$prefixdir" -maxdepth 1 -type f		\
	    -name "$prefixfile"\* -printf '%T@-%p\n'	\
	  | sort -rn					\
	  | sed "${nfilesd}s/^[0-9.,]*-//"		\
	  | while read f; do
#		echo "DEBUG: delfile='$f'"
		if [ "$mtime" = "" -o "$(find "$f" $mtime)" = "$f" ]; then
			if [ "$verbose" = "yes" ]; then
				echo >&2 "$MYSELF: Deleting '$f'."
			fi
			if [ "$pretend" = "no" ]; then
				rm -f "$f" || echo >&2 "$MYSELF: Error deleting '$f'."
			fi
		fi
	done
done
